#!/usr/bin/env python
# setup.py generated by flit for tools that don't yet use PEP 517

from distutils.core import setup

packages = \
['emojisearcher']

package_data = \
{'': ['*']}

install_requires = \
['pyperclip >=1.8.2', 'rich >=12.3.0']

extras_require = \
{'test': ['pytest']}

setup(name='emojisearcher',
      version='0.5.0',
      description='Look up emojis by text and copy them to the clipboard.',
      author=None,
      author_email='Bob Belderbos <bob@pybit.es>',
      url=None,
      packages=packages,
      package_data=package_data,
      install_requires=install_requires,
      extras_require=extras_require,
      python_requires='>=3.10',
     )
